from pathlib import Path

from _typeshed import Incomplete
from mastodon import Mastodon

from robotooter.mastodon.mixin import MastodonMixin as MastodonMixin

class Authenticator(MastodonMixin):
    main_path: Incomplete
    bot_path: Incomplete
    full_username: Incomplete
    def __init__(self, main_path: Path, bot_path: Path, full_username: str) -> None: ...
    def get_auth_url(self) -> str: ...
    def log_in(self, code: str) -> None: ...
    @property
    def client(self) -> Mastodon: ...
    def get_or_create_app(self) -> Mastodon: ...
    def create_app(self) -> None: ...
    @property
    def server_config_path(self) -> Path: ...
