from robotooter import RoboTooter as RoboTooter
from robotooter.cli.util import get_yes as get_yes

def get_options(text: str, options: dict[str, type], required: bool = False, only_one: bool = False) -> list[str]: ...
def run_create_bot(rt: RoboTooter) -> None: ...
