from _typeshed import Incomplete

from robotooter.filters.base_filter import BaseFilter as BaseFilter

USE_NL: Incomplete

class ParagraphCombiningFilter(BaseFilter):
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def process(self, line: str) -> str | None: ...
    def finalize(self) -> str | None: ...
