from pathlib import Path

class MastodonMixin:
    def split_username(self, full_username: str) -> tuple[str, str]: ...
    @property
    def bot_config_exists(self) -> bool: ...
    @property
    def bot_config_path(self) -> Path: ...
